#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import time
import os
import wx
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
import nardascripting.base.toolbox as tb


class IQStrCntr(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this code unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Base settings
        self._tab_name = "nsltools"
        self._scr_title = "IQ Stream Control V1.0"
        self._scr_description = "Setup an starts an IQ stream."
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_icon_close.png")
        self._list_prio = 0
        self._nsl_executed_behavior = NSL_Executed_Behaviors.MINIMIZE_NSL

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        # Connect to the device
        self.signalshark.connect()

        if not self.signalshark.scpi.check_add_task(TaskTypes.RT_STREAMING, "IQStream"):
            self.signalshark.scpi.stream.add(StrTypes.VITA49_IQ)
            self.signalshark.scpi.stream.set_select(1)
        else:
            self.signalshark.scpi.stream.set_select(1)

        self.signalshark.scpi.stream.connection_close()
        self.signalshark.scpi.stream.set_connection_idn(12345)
        self.signalshark.scpi.stream.set_connection_type(StrConTypes.UDP_SINGLECAST)

        cancel = False
        pc_ip = self.config.get("pc_ip", "192.168.128.4")
        dlg = wx.TextEntryDialog(self.frm_dlghelper, 'Enter IP address of PC', 'IP Address')
        dlg.SetValue(pc_ip)

        if self.ShowDlgModalTop(dlg) == wx.ID_OK:
            pc_ip = dlg.GetValue()
            self.config['pc_ip'] = pc_ip
        else:
            cancel = True
        dlg.Destroy()
        if cancel:
            return

        iq_cbw = self.config.get('iq_cbw', 10.0)
        dlg = wx.NumberEntryDialog(self.frm_dlghelper, 'Enter IQ streaming bandwidth',
                                   'CBW', 'IQ Streaming CBW', iq_cbw, 1.0, 40.0)
        if self.ShowDlgModalTop(dlg) == wx.ID_OK:
            iq_cbw = dlg.GetValue()
            self.config['iq_cbw'] = iq_cbw
        else:
            cancel = True
        dlg.Destroy()
        if cancel:
            return

        self.signalshark.scpi.stream.set_connection_address(pc_ip)
        self.signalshark.scpi.stream.set_connection_port(4444)

        self.signalshark.scpi.iqstream.set_oversampling(False)
        self.signalshark.scpi.iqstream.set_payload_limit(2048)
        self.signalshark.scpi.iqstream.set_payload_format(StrPayForm.I16Q16)
        self.signalshark.scpi.iqstream.set_frequency_tune(90e6)
        self.signalshark.scpi.iqstream.set_cbw(iq_cbw * 1e6)
        self.signalshark.scpi.stream.connection_open()

        if self.signalshark.scpi.check_error():
            self.main_gui.MessageBoxModal("Error while initialization.", "Initialization failed",
                                          wx.OK | wx.ICON_INFORMATION)

        # Disconnect from the device
        self.signalshark.disconnect()
